@@ -193,7 +193,7 @@ int main(int argc, char* argv[])
     enum eTerminalKanjiCode code = kTKUtf8;
 
     char cwd[PATH_MAX];
-    mygetcwd(cwd);
+    mygetcwd(cwd, PATH_MAX);
     setenv("PWD", cwd, 1);
 
     vector_obj* argv2 = VECTOR_NEW(10);
