@@ -232,22 +232,22 @@ static BOOL read_one_argument(char* p, string_obj* buf)
     }
 }
 
-void parentname2(char* result, char* path)
+void parentname2(char* result, int result_size, char* path)
 {
     char* p;
 
     if(*path == 0) {
-        strcpy(result, "");
+        strncpy(result, "", result_size);
         return;
     }
     
     if(strcmp(path, "/") == 0) {
-        strcpy(result, "/");
+        strncpy(result, "/", result_size);
         return;
     }
 
     for(p=path + strlen(path)-1; p != path-1; p--) {
-        if(*p == '/' ) {
+        if(*p == '/') {
             memcpy(result, path, p-path);
             result[p-path] = '/';
             result[p-path+1] = 0;
@@ -256,7 +256,7 @@ void parentname2(char* result, char* path)
         }
     }
 
-    strcpy(result, "");
+    strncpy(result, "", result_size);
 }
 
 
@@ -379,7 +379,7 @@ char* readline_file_completion_migemo_match(char* text, char* file)
     /// text側がfileの部分なら判定するまでも無くマッチ ///
     else if(same_point == strlen(text)) {
         char tmp[PATH_MAX];
-        strcpy(tmp, file);
+        strncpy(tmp, file, PATH_MAX);
         string_obj* tmp2 = STRING_NEW("");
         saphire_get_quoted_fname_readline_ver(tmp, tmp2);
         char* ret = strdup(string_c_str(tmp2));
@@ -453,7 +453,7 @@ char* readline_file_completion_migemo_match(char* text, char* file)
 
     if(r2 == 0) {
         char tmp[PATH_MAX];
-        strcpy(tmp, file);
+        strncpy(tmp, file, PATH_MAX);
         string_obj* tmp2 = STRING_NEW("");
         saphire_get_quoted_fname_readline_ver(tmp, tmp2);
         char* ret = strdup(string_c_str(tmp2));
@@ -494,7 +494,7 @@ char* readline_file_completion_migemo(const char* text, int rstat)
         string_put(gEditingDir, "");
         read_one_argument(rl_line_buffer, gEditingDir);
         char* tmp = MALLOC(string_length(gEditingDir) + 1);
-        parentname2(tmp, string_c_str(gEditingDir));
+        parentname2(tmp, string_length(gEditingDir) + 1, string_c_str(gEditingDir));
         string_put(gEditingDir, tmp);
         FREE(tmp);
 
@@ -570,34 +570,34 @@ char* readline_file_completion_migemo(const char* text, int rstat)
                         char buf[PATH_MAX];
 
                         struct stat _stat;
-                        strcpy(buf, string_c_str(gEditingDir));
-                        strcat(buf, entry->d_name);
+                        strncpy(buf, string_c_str(gEditingDir), PATH_MAX);
+                        strncat(buf, entry->d_name, PATH_MAX);
                         stat(buf, &_stat);
 
 #if defined(__DARWIN__)
                         {
                         char* src = entry->d_name;
-                        char* des = MALLOC(strlen(src)*2);
                         int des_size = strlen(src)*2;
+                        char* des = MALLOC(des_size);
                         if(is_all_ascii(src)) {
-                            strcpy(des, src);
+                            strncpy(des, src, des_size);
                         }
                         else {
                             /// コンバート ///
                             if(kanji_convert(src, des, des_size, kUtf8Mac, kUtf8) < 0) {
-                                strcpy(des, src);
+                                strncpy(des, src, des_size);
                             }
                         }
-                        strcpy(buf, des);
+                        strncpy(buf, des, PATH_MAX);
 
                         FREE(des);
                         }
 #else
-                        strcpy(buf, entry->d_name);
+                        strncpy(buf, PATH_MAX, entry->d_name);
 #endif
 
                         if(S_ISDIR(_stat.st_mode)) {
-                            strcat(buf, "/");
+                            strncat(buf, "/", PATH_MAX);
                         }
 
                         vector_add(files, STRING_NEW(buf));
@@ -621,32 +621,32 @@ char* readline_file_completion_migemo(const char* text, int rstat)
                         char buf[PATH_MAX];
 
                         struct stat _stat;
-                        strcpy(buf, "./");
-                        strcat(buf, entry->d_name);
+                        strncpy(buf, "./", PATH_MAX);
+                        strncat(buf, entry->d_name, PATH_MAX);
                         stat(buf, &_stat);
 
 #if defined(__DARWIN__)
                         {
                         char* src = entry->d_name;
-                        char* des = MALLOC(strlen(src)*2);
                         int des_size = strlen(src)*2;
+                        char* des = MALLOC(des_size);
                         if(is_all_ascii(src)) {
-                            strcpy(des, src);
+                            strncpy(des, src, des_size);
                         }
                         else {
                             /// コンバート ///
                             if(kanji_convert(src, des, des_size, kUtf8Mac, kUtf8) < 0) {
-                                strcpy(des, src);
+                                strncpy(des, src, des_size);
                             }
                         }
-                        strcpy(buf, des);
+                        strncpy(buf, des, PATH_MAX);
                         FREE(des);
                         }
 #else
-                        strcpy(buf, entry->d_name);
+                        strncpy(buf, PATH_MAX, entry->d_name);
 #endif
                         if(S_ISDIR(_stat.st_mode)) {
-                            strcat(buf, "/");
+                            strncat(buf, "/", PATH_MAX);
                         }
                         vector_add(files, STRING_NEW(buf));
                     }
@@ -666,7 +666,7 @@ char* readline_file_completion_migemo(const char* text, int rstat)
         /// ファイル名の入力がない補完は判定しなくても全てマッチ ///
         if(strcmp(text2, "") == 0) {
             char path[PATH_MAX];
-            strcpy(path, file);
+            strncpy(path, file, PATH_MAX);
 
             string_obj* tmp2 = STRING_NEW("");
             saphire_get_quoted_fname_readline_ver(path, tmp2);
@@ -674,8 +674,8 @@ char* readline_file_completion_migemo(const char* text, int rstat)
             string_delete(tmp2);
 
             struct stat stat_;
-            strcpy(path, string_c_str(gEditingDir));
-            strcat(path, file);
+            strncpy(path, string_c_str(gEditingDir), PATH_MAX);
+            strncat(path, file, PATH_MAX);
             if(stat(path, &stat_) == 0) {
                 if(S_ISDIR(stat_.st_mode)) {
                     rl_completion_append_character= 0;
@@ -698,15 +698,15 @@ char* readline_file_completion_migemo(const char* text, int rstat)
         else if(is_all_ascii((char*)text2) && is_all_ascii(file)) {
             if(!strncmp(text2, file, wordlen)) {
                 char path[PATH_MAX];
-                strcpy(path, file);
+                strncpy(path, file, PATH_MAX);
 
                 string_obj* tmp2 = STRING_NEW("");
                 saphire_get_quoted_fname(path, tmp2);
                 char* ret = strdup(string_c_str(tmp2));
                 string_delete(tmp2);
 
-                strcpy(path, string_c_str(gEditingDir));
-                strcat(path, file);
+                strncpy(path, string_c_str(gEditingDir), PATH_MAX);
+                strncat(path, file, PATH_MAX);
                 struct stat stat_;
                 if(stat(path, &stat_) == 0) {
                     if(S_ISDIR(stat_.st_mode)) {
@@ -732,8 +732,8 @@ char* readline_file_completion_migemo(const char* text, int rstat)
             if(ret) {
                 struct stat stat_;
                 char path[PATH_MAX];
-                strcpy(path, string_c_str(gEditingDir));
-                strcat(path, file);
+                strncpy(path, string_c_str(gEditingDir), PATH_MAX);
+                strncat(path, file, PATH_MAX);
                 if(stat(path, &stat_) == 0) {
                     if(S_ISDIR(stat_.st_mode)) {
                         rl_completion_append_character= 0;
@@ -799,23 +799,24 @@ char* readline_env_completion(const char* text, int stat)
     while(1) {
         if(index < vector_size(gSaphireVarCompletions)) {
             char* env = string_c_str(vector_item(gSaphireVarCompletions, index));
-            char* env2 = malloc(strlen(env) + 10);
+            int env2_size = strlen(env) + 10;
+            char* env2 = malloc(env2_size);
 
             if(*text == '$' && *(text+1) != '$') {
-                strcpy(env2, "$");
-                strcat(env2, env);
+                strncpy(env2, "$", env2_size);
+                strncat(env2, env, env2_size);
             }
             else if(*text == '$' && *(text+1) == '$') {
-                strcpy(env2, "$$");
-                strcat(env2, env);
+                strncpy(env2, "$$", env2_size);
+                strncat(env2, env, env2_size);
             }
             else if(*text == '@' && *(text+1) != '@') {
-                strcpy(env2, "@");
-                strcat(env2, env);
+                strncpy(env2, "@", env2_size);
+                strncat(env2, env, env2_size);
             }
             else if(*text == '@' && *(text+1) == '@') {
-                strcpy(env2, "@@");
-                strcat(env2, env);
+                strncpy(env2, "@@", env2_size);
+                strncat(env2, env, env2_size);
             }
 
             index++;
@@ -862,7 +863,7 @@ char* readline_file_completion(const char* text, int rstat)
         string_put(gEditingDir, "");
         read_one_argument(rl_line_buffer, gEditingDir);
         char* tmp = MALLOC(string_length(gEditingDir) + 1);
-        parentname2(tmp, string_c_str(gEditingDir));
+        parentname2(tmp, string_length(gEditingDir) + 1, string_c_str(gEditingDir));
         string_put(gEditingDir, tmp);
         FREE(tmp);
 
@@ -938,32 +939,32 @@ char* readline_file_completion(const char* text, int rstat)
                         char buf[PATH_MAX];
 
                         struct stat _stat;
-                        strcpy(buf, string_c_str(gEditingDir));
-                        strcat(buf, entry->d_name);
+                        strncpy(buf, string_c_str(gEditingDir), PATH_MAX);
+                        strncat(buf, entry->d_name, PATH_MAX);
                         stat(buf, &_stat);
 
 #if defined(__DARWIN__)
                         {
                         char* src = entry->d_name;
-                        char* des = MALLOC(strlen(src)*2);
                         int des_size = strlen(src)*2;
+                        char* des = MALLOC(des_size);
                         if(is_all_ascii(src)) {
-                            strcpy(des, src);
+                            strncpy(des, src, des_size);
                         }
                         else {
                             /// コンバート ///
                             if(kanji_convert(src, des, des_size, kUtf8Mac, kUtf8) < 0) {
-                                strcpy(des, src);
+                                strncpy(des, src, des_size);
                             }
                         }
-                        strcpy(buf, des);
+                        strncpy(buf, des, PATH_MAX);
                         FREE(des);
                         }
 #else
-                        strcpy(buf, entry->d_name);
+                        strncpy(buf, entry->d_name, PATH_MAX);
 #endif
                         if(S_ISDIR(_stat.st_mode)) {
-                            strcat(buf, "/");
+                            strncat(buf, "/", PATH_MAX);
                         }
 
                         vector_add(files, STRING_NEW(buf));
@@ -987,32 +988,32 @@ char* readline_file_completion(const char* text, int rstat)
                         char buf[PATH_MAX];
 
                         struct stat _stat;
-                        strcpy(buf, "./");
-                        strcat(buf, entry->d_name);
+                        strncpy(buf, "./", PATH_MAX);
+                        strncat(buf, entry->d_name, PATH_MAX);
                         stat(buf, &_stat);
 
 #if defined(__DARWIN__)
                         {
                         char* src = entry->d_name;
-                        char* des = MALLOC(strlen(src)*2);
                         int des_size = strlen(src)*2;
+                        char* des = MALLOC(des_size);
                         if(is_all_ascii(src)) {
-                            strcpy(des, src);
+                            strncpy(des, src, des_size);
                         }
                         else {
                             /// コンバート ///
                             if(kanji_convert(src, des, des_size, kUtf8Mac, kUtf8) < 0) {
-                                strcpy(des, src);
+                                strncpy(des, src, des_size);
                             }
                         }
-                        strcpy(buf, des);
+                        strncpy(buf, des, PATH_MAX);
                         FREE(des);
                         }
 #else
-                        strcpy(buf, entry->d_name);
+                        strncpy(buf, entry->d_name, PATH_MAX);
 #endif
                         if(S_ISDIR(_stat.st_mode)) {
-                            strcat(buf, "/");
+                            strncat(buf, "/", PATH_MAX);
                         }
                         vector_add(files, STRING_NEW(buf));
                     }
@@ -1032,7 +1033,7 @@ char* readline_file_completion(const char* text, int rstat)
         /// ファイル名の入力がない補完は判定しなくても全てマッチ ///
         if(strcmp(text2, "") == 0) {
             char path[PATH_MAX];
-            strcpy(path, file);
+            strncpy(path, file, PATH_MAX);
 
             string_obj* tmp2 = STRING_NEW("");
             saphire_get_quoted_fname_readline_ver(path, tmp2);
@@ -1040,8 +1041,8 @@ char* readline_file_completion(const char* text, int rstat)
             string_delete(tmp2);
 
             struct stat stat_;
-            strcpy(path, string_c_str(gEditingDir));
-            strcat(path, file);
+            strncpy(path, string_c_str(gEditingDir),  PATH_MAX);
+            strncat(path, file, PATH_MAX);
             if(stat(path, &stat_) == 0) {
                 if(S_ISDIR(stat_.st_mode)) {
                     rl_completion_append_character= 0;
@@ -1065,15 +1066,15 @@ char* readline_file_completion(const char* text, int rstat)
         else {
             if(!strncmp(text2, file, wordlen)) {
                 char path[PATH_MAX];
-                strcpy(path, file);
+                strncpy(path, file, PATH_MAX);
 
                 string_obj* tmp2 = STRING_NEW("");
                 saphire_get_quoted_fname(path, tmp2);
                 char* ret = strdup(string_c_str(tmp2));
                 string_delete(tmp2);
 
-                strcpy(path, string_c_str(gEditingDir));
-                strcat(path, file);
+                strncpy(path, string_c_str(gEditingDir), PATH_MAX);
+                strncat(path, file, PATH_MAX);
                 struct stat stat_;
                 if(stat(path, &stat_) == 0) {
                     if(S_ISDIR(stat_.st_mode)) {
@@ -2046,8 +2047,8 @@ static BOOL readline_select_file_read(vector_obj* v, char* path)
     struct dirent* entry;
     while(entry = readdir(dir)) {
         char fpath[PATH_MAX];
-        strcpy(fpath, path);
-        strcat(fpath, entry->d_name);
+        strncpy(fpath, path, PATH_MAX);
+        strncat(fpath, entry->d_name, PATH_MAX);
 
         struct stat stat_;
         memset(&stat_, 0, sizeof(struct stat));
@@ -2068,6 +2069,85 @@ static BOOL readline_select_file_read(vector_obj* v, char* path)
     return TRUE;
 }
 
+static void str_cut2(enum eKanjiCode code, char* mbs, int termsize, char* dest_mbs, int dest_byte)
+{
+    if(code == kUtf8) {
+        int i;
+        int n;
+
+        wchar_t* wcs 
+            = (wchar_t*)MALLOC(sizeof(wchar_t)*(termsize+1)*MB_CUR_MAX);
+        wchar_t* tmp 
+            = (wchar_t*)MALLOC(sizeof(wchar_t)*(termsize+1)*MB_CUR_MAX);
+
+        if(mbstowcs(wcs, mbs, (termsize+1)*MB_CUR_MAX) == -1) {
+            mbstowcs(wcs, "?????", (termsize+1)*MB_CUR_MAX);
+        }
+
+        i = 0;
+        while(1) {
+            if(i < wcslen(wcs)) {
+                tmp[i] = wcs[i];
+                tmp[i+1] = 0;
+            }
+            else {
+                tmp[i] = ' ';
+                tmp[i+1] = 0;
+            }
+
+            n = wcswidth(tmp, wcslen(tmp));
+            if(n < 0) {
+                strncpy(dest_mbs, "?????", dest_byte);
+
+                FREE(wcs);
+                FREE(tmp);
+
+                return;
+            }
+            else {
+                if(n > termsize) {
+                    tmp[i] = 0;
+
+                    if(wcswidth(tmp, wcslen(tmp)) != termsize) {
+                        tmp[i] = ' ';
+                        tmp[i+1] = 0;
+                    }
+                    break;
+                }
+            }
+
+            i++;
+        }
+
+        wcstombs(dest_mbs, tmp, dest_byte);
+
+        FREE(wcs);
+        FREE(tmp);
+    }
+    else {
+        int n;
+        BOOL tmpend = FALSE;
+        BOOL kanji = FALSE;
+        for(n=0; n<termsize && n<dest_byte-1; n++) {
+            if(kanji)
+                kanji = FALSE;
+            else if(!tmpend && is_kanji(code, mbs[n])) {
+                kanji = TRUE;
+            }
+
+            if(!tmpend && mbs[n] == 0) tmpend = TRUE;
+                        
+            if(tmpend)
+                dest_mbs[n] = ' ';
+            else
+                dest_mbs[n] = mbs[n];
+        }
+        
+        if(kanji) dest_mbs[n-1] = ' ';
+        dest_mbs[n] = 0;
+    }
+}
+
 
 static int readline_select_file(int count, int key)
 {
@@ -2080,11 +2160,11 @@ static int readline_select_file(int count, int key)
 
     vector_obj* v = VECTOR_NEW(10);
     char cwd_path[PATH_MAX];
-    mygetcwd(cwd_path);
-    strcat(cwd_path, "/");
+    mygetcwd(cwd_path, PATH_MAX);
+    strncat(cwd_path, "/", PATH_MAX);
 
     char current_dir[PATH_MAX];
-    strcpy(current_dir, cwd_path);
+    strncpy(current_dir, cwd_path, PATH_MAX);
     readline_select_file_read(v, current_dir);
 
     vector_obj* mark_files = VECTOR_NEW(10);
@@ -2119,8 +2199,8 @@ static int readline_select_file(int count, int key)
                 int x = ((n-page*4*(maxy-1)) % 4)*maxx/4;
 
                 char full_path[PATH_MAX];
-                strcpy(full_path, current_dir);
-                strcat(full_path, string_c_str(file->mName));
+                strncpy(full_path, current_dir, PATH_MAX);
+                strncat(full_path, string_c_str(file->mName), PATH_MAX);
 
                 BOOL marked = FALSE;
                 int i;
@@ -2178,8 +2258,8 @@ static int readline_select_file(int count, int key)
                 string_obj* cursor_mfile = NULL;
 
                 char cursor_path[PATH_MAX];
-                strcpy(cursor_path, current_dir);
-                strcat(cursor_path, string_c_str(cursor_file->mName));
+                strncpy(cursor_path, current_dir, PATH_MAX);
+                strncat(cursor_path, string_c_str(cursor_file->mName), PATH_MAX);
 
                 int i;
                 for(i=0; i<vector_size(mark_files); i++) {
@@ -2203,7 +2283,7 @@ static int readline_select_file(int count, int key)
                 cursor++;
             }
             else if(key == '\\') {
-                strcpy(current_dir, "/");
+                strncpy(current_dir, "/", PATH_MAX);
 
                 int i;
                 for(i=0; i<vector_size(v); i++) {
@@ -2221,8 +2301,8 @@ static int readline_select_file(int count, int key)
                     /// 現在のカレントディレクトリのパスなら
                     /// ディレクトリ名を取る
                     char cursor_path[PATH_MAX];
-                    strcpy(cursor_path, current_dir);
-                    strcat(cursor_path, string_c_str(cursor_file->mName));
+                    strncpy(cursor_path, current_dir, PATH_MAX);
+                    strncat(cursor_path, string_c_str(cursor_file->mName), PATH_MAX);
 
                     /// カレントのファイル、子ディレクトリのファイルなら
                     /// 相対ディレクトリ
@@ -2250,8 +2330,9 @@ static int readline_select_file(int count, int key)
                     int j;
                     for(j=0; j<vector_size(mark_files); j++) {
                         char full_path[PATH_MAX];
-                        strcpy(full_path
-                            , string_c_str(vector_item(mark_files, j)));
+                        strncpy(full_path
+                            , string_c_str(vector_item(mark_files, j))
+                            , PATH_MAX);
 
                         if(strstr(full_path, cwd_path) == full_path) {
                             char full_path2[PATH_MAX];
@@ -2293,11 +2374,12 @@ static int readline_select_file(int count, int key)
                 {
                     char path2[PATH_MAX];
                     if(correct_path(current_dir
-                            , string_c_str(cursor_file->mName), path2)) 
+                            , string_c_str(cursor_file->mName), path2, PATH_MAX)) 
                     {
-                        strcpy(current_dir, path2);
+                        strncpy(current_dir, path2
+                            , PATH_MAX);
                         if(current_dir[strlen(current_dir)-1] != '/') {
-                            strcat(current_dir, "/");
+                            strncat(current_dir, "/", PATH_MAX);
                         }
 
                         int i;
@@ -2313,8 +2395,8 @@ static int readline_select_file(int count, int key)
             }
             else if(key == 8 || key == KEY_BACKSPACE) {    // CTRL-H
                 char parent_path[PATH_MAX];
-                parentname(parent_path, current_dir);
-                strcpy(current_dir, parent_path);
+                parentname(parent_path, PATH_MAX, current_dir);
+                strncpy(current_dir, parent_path, PATH_MAX);
 
                 int i;
                 for(i=0; i<vector_size(v); i++) {
