@@ -312,7 +312,7 @@ void saphire_sweep();
     // ごみ掃除
     // sapire_shellやsaphire_loadが呼ばれた後呼び出してください
 
-vector_obj* gSaphireVarCompletions;
+extern vector_obj* gSaphireVarCompletions;
 // 変数補完候補
 
 void saphire_make_var_completion_list();          
