@@ -19,67 +19,67 @@
 //////////////////////////////////////////////////////////////////////
 // ?g???q?????Ԃ?
 //////////////////////////////////////////////////////////////////////
-void extname(SPACE char* result, char* name)
+void extname(SPACE char* result, int result_size, char* name)
 {
     char* p;
     
     for(p = name + strlen(name); p != name; p--) {
         if(*p == '.' && p == name + strlen(name)) {
-            strcpy(result, "");
+            strncpy(result, "", result_size);
             return;
         }
         else if(*p == '/') {
-            strcpy(result, "");
+            strncpy(result, "", result_size);
             return;
         }
         else if(*p == '.') {
-            strcpy(result, p+1);
+            strncpy(result, p+1, result_size);
             return;
         }
     }
 
-    strcpy(result, "");
+    strncpy(result, "", result_size);
 }
 
 //////////////////////////////////////////////////////////////////////
 // ?g???q???ȊO???Ԃ?
 //////////////////////////////////////////////////////////////////////
-void noextname(SPACE char* result, char* name)
+void noextname(SPACE char* result, int result_size, char* name)
 {
     char* max;
     char* p;
     
     max = name + strlen(name);
     for(p = name; p < max; p++) {
-        if(*p == '.') {
+        if(*p == '.' && (p - name + 1) < result_size) {
             memcpy(result, name, p - name);
             result[p - name] = 0;
             return;
         }
     }
 
-    strcpy(result, name);
+    strncpy(result, name, result_size);
 }
 
 //////////////////////////////////////////////////////////////////////
 // ?e?f?B???N?g???????Ԃ?
 //////////////////////////////////////////////////////////////////////
-void parentname(SPACE char* result, char* path)
+void parentname(SPACE char* result, int result_size, char* path)
 {
     char* p;
 
     if(*path == 0) {
-        strcpy(result, "");
+        strncpy(result, "", result_size);
         return;
     }
     
     if(strcmp(path, "/") == 0) {
-        strcpy(result, "/");
+        strncpy(result, "/", result_size);
         return;
     }
 
     for(p=path + strlen(path)-2; p != path-1; p--) {
-        if(*p == '/' ) {
+        if(*p == '/' && (p-path + 2) < result_size) {
             memcpy(result, path, p-path);
             result[p-path] = '/';
             result[p-path+1] = 0;
@@ -88,65 +88,25 @@ void parentname(SPACE char* result, char* path)
         }
     }
 
-    strcpy(result, "");
+    strncpy(result, "", result_size);
 }
 
 //////////////////////////////////////////////////////////////////////
 // ?J?????g?f?B???N?g?????Ԃ?
 //////////////////////////////////////////////////////////////////////
-void mygetcwd(SPACE char* result)
+void mygetcwd(SPACE char* result, int result_size)
 {
     int l;
     
     l = 50;
     while(!getcwd(result, l)) {
          l += 50;
+         if(l+1 >= result_size) {
+            break;
+         }
     }
 }
 
-
-//////////////////////////////////////////////////////////////////////
-// ?????????????Ԃ?
-//////////////////////////////////////////////////////////////////////
-char* mystrcasestr(const char *haystack, const char *needle)
-{
-    char* p;
-    p = (char*)haystack;
-
-    while(*p) {
-        char* p2;
-        char* p3;
-        BOOL match;
-
-        p2 = p;
-        p3 = (char*)needle;
-        
-        match = TRUE;
-        while(*p3) {
-            if(*p2 == 0) {
-                match = FALSE;
-                break;
-            }
-            
-            if(tolower(*p2) != tolower(*p3)) {
-                match = FALSE;
-                break;
-            }
-            
-            p2++;
-            p3++;
-        }
-        
-        if(match) {
-            return p;
-        }
-        
-        p++;
-    }
-    
-    return NULL;
-}
-
 //////////////////////////////////////////////////////////////////////
 // ?????????S?????????ɂ???
 //////////////////////////////////////////////////////////////////////
@@ -163,7 +123,7 @@ void mystrtolower(char* result)
 //////////////////////////////////////////////////////////////////////
 // パスを絶対パスにする
 //////////////////////////////////////////////////////////////////////
-int correct_path(char* current_path, char* path, char* path2)
+int correct_path(char* current_path, char* path, char* path2, int path2_size)
 {
     char tmp[PATH_MAX];
     char tmp2[PATH_MAX];
@@ -172,30 +132,30 @@ int correct_path(char* current_path, char* path, char* path2)
 
     /// 先頭にカレントパスを加える ///
     {
-        if(path[0] == '/') {
-            strcpy(tmp, path);
+        if(path[0] == '/') {      /// 絶対パスなら必要ない
+            strncpy(tmp, path, PATH_MAX);
         }
         else {
             if(current_path == NULL) {
                 char cwd[PATH_MAX];
-                mygetcwd(cwd);
+                mygetcwd(cwd, PATH_MAX);
                 
-                strcpy(tmp, cwd);
-                strcat(tmp, "/");
-                strcat(tmp, path);
+                strncpy(tmp, cwd, PATH_MAX);
+                strncat(tmp, "/", PATH_MAX);
+                strncat(tmp, path, PATH_MAX);
             }
             else {
-                strcpy(tmp, current_path);
+                strncpy(tmp, current_path, PATH_MAX);
                 if(current_path[strlen(current_path)-1] != '/') {
-                    strcat(tmp, "/");
+                    strncat(tmp, "/", PATH_MAX);
                 }
-                strcat(tmp, path);
+                strncat(tmp, path, PATH_MAX);
             }
         }
 
     }
 
-    strcpy(tmp2, tmp);
+    strncpy(tmp2, tmp, PATH_MAX);
 
     /// .を削除する ///
     {
@@ -240,7 +200,7 @@ int correct_path(char* current_path, char* path, char* path2)
                     p--;
 
                     if(p < tmp4) {
-                        strcpy(path2, "/");
+                        strncpy(path2, "/", path2_size);
                         return FALSE;
                     }
                 } while(*p != '/');
@@ -254,7 +214,7 @@ int correct_path(char* current_path, char* path, char* path2)
                     p--;
 
                     if(p < tmp4) {
-                        strcpy(path2, "/");
+                        strncpy(path2, "/", path2_size);
                         return FALSE;
                     }
                 } while(*p != '/');
@@ -270,12 +230,54 @@ int correct_path(char* current_path, char* path, char* path2)
     }
 
     if(*tmp4 == 0) {
-        strcpy(path2, "/");
+        strncpy(path2, "/", path2_size);
     }
     else {
-        strcpy(path2, tmp4);
+        strncpy(path2, tmp4, path2_size);
     }
 
     return TRUE;
 }
 
+//////////////////////////////////////////////////////////////////////
+// ?????????????Ԃ?
+//////////////////////////////////////////////////////////////////////
+char* mystrcasestr(const char *haystack, const char *needle)
+{
+    char* p;
+    p = (char*)haystack;
+
+    while(*p) {
+        char* p2;
+        char* p3;
+        BOOL match;
+
+        p2 = p;
+        p3 = (char*)needle;
+        
+        match = TRUE;
+        while(*p3) {
+            if(*p2 == 0) {
+                match = FALSE;
+                break;
+            }
+            
+            if(tolower(*p2) != tolower(*p3)) {
+                match = FALSE;
+                break;
+            }
+            
+            p2++;
+            p3++;
+        }
+        
+        if(match) {
+            return p;
+        }
+        
+        p++;
+    }
+    
+    return NULL;
+}
+
