@@ -2671,7 +2671,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
 
     gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "1.1.7", 1);
+    setenv("SAPHIRE_VERSION", "1.2.0", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
@@ -2740,9 +2740,9 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
 
     char* env = getenv("SAPHIRE_TMPDIR");
     if(env) 
-        strcpy(gTmpDir, env);
+        strncpy(gTmpDir, env, PATH_MAX);
     else 
-        strcpy(gTmpDir, "/tmp");
+        strncpy(gTmpDir, "/tmp", PATH_MAX);
 
     gMatchRestart = HASH_NEW(1000);
     gStackTraceFName = VECTOR_NEW(30);
