@@ -19,7 +19,7 @@ string_obj* string_new_debug(char* str, const char* fname, int line, const char*
     
     self->mMallocLen = (len < 128) ? 128 : (len + 1) * 2;
     self->mStr = (char*)MALLOC(sizeof(char)*self->mMallocLen);
-    strcpy(self->mStr, str);
+    strncpy(self->mStr, str, self->mMallocLen);
     
     self->mLen = len;
 
@@ -35,7 +35,7 @@ string_obj* string_new(char* str)
     
     self->mMallocLen = (len < 128) ? 128 : (len + 1) * 2;
     self->mStr = (char*)MALLOC(sizeof(char)*self->mMallocLen);
-    strcpy(self->mStr, str);
+    strncpy(self->mStr, str, self->mMallocLen);
     
     self->mLen = len;
 
